@@ -41,6 +41,21 @@ const_iv(MISALIGNED_SUB_BUFFER_OFFSET)
 #ifdef CL_EXEC_STATUS_ERROR_FOR_EVENTS_IN_WAIT_LIST
 const_iv(EXEC_STATUS_ERROR_FOR_EVENTS_IN_WAIT_LIST)
 #endif
+#ifdef CL_COMPILE_PROGRAM_FAILURE
+const_iv(COMPILE_PROGRAM_FAILURE)
+#endif
+#ifdef CL_LINKER_NOT_AVAILABLE
+const_iv(LINKER_NOT_AVAILABLE)
+#endif
+#ifdef CL_LINK_PROGRAM_FAILURE
+const_iv(LINK_PROGRAM_FAILURE)
+#endif
+#ifdef CL_DEVICE_PARTITION_FAILED
+const_iv(DEVICE_PARTITION_FAILED)
+#endif
+#ifdef CL_KERNEL_ARG_INFO_NOT_AVAILABLE
+const_iv(KERNEL_ARG_INFO_NOT_AVAILABLE)
+#endif
 #ifdef CL_INVALID_VALUE
 const_iv(INVALID_VALUE)
 #endif
@@ -146,6 +161,18 @@ const_iv(INVALID_GLOBAL_WORK_SIZE)
 #ifdef CL_INVALID_PROPERTY
 const_iv(INVALID_PROPERTY)
 #endif
+#ifdef CL_INVALID_IMAGE_DESCRIPTOR
+const_iv(INVALID_IMAGE_DESCRIPTOR)
+#endif
+#ifdef CL_INVALID_COMPILER_OPTIONS
+const_iv(INVALID_COMPILER_OPTIONS)
+#endif
+#ifdef CL_INVALID_LINKER_OPTIONS
+const_iv(INVALID_LINKER_OPTIONS)
+#endif
+#ifdef CL_INVALID_DEVICE_PARTITION_COUNT
+const_iv(INVALID_DEVICE_PARTITION_COUNT)
+#endif
 #ifdef CL_PLATFORM_NOT_FOUND_KHR
 const_iv(PLATFORM_NOT_FOUND_KHR)
 #endif
